#include <qlayout.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdir.h> 

#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <kdialog.h>
#include <kcombobox.h>
#include <kstandarddirs.h>
#include <kurlrequester.h>
#include <kdialogbase.h>

#include <klineedit.h>
#include <knuminput.h>

#include <pluginconf.h>

#include "hadifaxspeech.h"
#include "voicefileui.h"

#include "hadifaxconfigui.h"
#include "hadifaxconf.h"
#include "hadifaxconf.moc"

class HadifaxConfPrivate {
   friend class HadifaxConf;
   private:
      HadifaxConfPrivate() {
         findInitialConfig();
      };
      
      ~HadifaxConfPrivate() {
      };
      
      #include "initialconfig.h"

      void setConfiguration (QString hadifaxExec,  QString mbrolaExec,
                             QString voice,        bool male,
                             int volume, int time, int pitch)
      {
         configWidget->hadifaxURL->setURL (hadifaxExec);
         configWidget->mbrolaURL->setURL (mbrolaExec);
         configWidget->setVoice (voice, male);
         
         configWidget->volumeBox->setValue (volume);
         configWidget->timeBox->setValue (time);
         configWidget->frequencyBox->setValue (pitch);
      }

      void initializeVoices () {
         QStringList::iterator it;
         for (it = defaultVoices.begin(); it != defaultVoices.end(); ++it) {
            HadifaxSpeech::VoiceGender gender;
            QString name = QFileInfo(*it).fileName();
            gender = HadifaxSpeech::determineGender(defaultMbrolaExec, *it);
            if (gender == HadifaxSpeech::MaleGender)
               configWidget->addVoice(*it, true, i18n("Male voice \""+name+"\""));
            else if (gender == HadifaxSpeech::FemaleGender)
               configWidget->addVoice(*it, false, i18n("Female voice \""+name+"\""));
            else {
               if (name == "de1")
                  configWidget->addVoice(*it, false, i18n("Female voice \""+name+"\""));
               else {
                  configWidget->addVoice(*it, true,  i18n("Unknown voice \""+name+"\""));
                  configWidget->addVoice(*it, false, i18n("Unknown voice \""+name+"\""));
               }
            }
         }
      };

      void setDefaults () {
         QStringList::iterator it = defaultVoices.begin();
         HadifaxSpeech::VoiceGender gender;
         gender = HadifaxSpeech::determineGender(defaultMbrolaExec, *it);
         
         setConfiguration (defaultHadifaxExec, defaultMbrolaExec,
                           *it, gender == HadifaxSpeech::MaleGender,
                           100, 100, 100);
      };

      void load (KConfig *config, const QString &langGroup) {
         config->setGroup(langGroup);
         
         QStringList::iterator it = defaultVoices.begin();
         QString voice = config->readEntry("voice", *it);
         
         HadifaxSpeech::VoiceGender gender;
         gender = HadifaxSpeech::determineGender(defaultMbrolaExec, voice);
         bool isMale = (gender == HadifaxSpeech::MaleGender);
         
         setConfiguration (
               config->readEntry ("hadifaxExec",defaultHadifaxExec),
               config->readEntry ("mbrolaExec", defaultMbrolaExec),
               config->readEntry ("voice",      voice),
               config->readBoolEntry("gender",  isMale),
               config->readNumEntry ("volume",  100),
               config->readNumEntry ("time",    100),
               config->readNumEntry ("pitch",   100)
         );
      };

      void save (KConfig *config, const QString &langGroup) {
         config->setGroup(langGroup);
         config->writeEntry ("hadifaxExec",configWidget->hadifaxURL->url());
         config->writeEntry ("mbrolaExec", configWidget->mbrolaURL->url());
         config->writeEntry ("voice",      configWidget->getVoiceFilename());
         config->writeEntry ("gender",     configWidget->isMaleVoice());
         config->writeEntry ("volume",     configWidget->volumeBox->value());
         config->writeEntry ("time",       configWidget->timeBox->value());
         config->writeEntry ("pitch",      configWidget->frequencyBox->value());
      }

      HadifaxConfigUI *configWidget;
      
      QString defaultHadifaxExec;
      QString defaultMbrolaExec;
      QStringList defaultVoices;
};

/** Constructor */
HadifaxConf::HadifaxConf( QWidget* parent, const char* name, const QStringList &) : 
   PlugInConf( parent, name ){
   kdDebug() << "Running: HadifaxConf::HadifaxConf( QWidget* parent, const char* name, const QStringList &args)" << endl;
   QVBoxLayout *layout = new QVBoxLayout (this, KDialog::marginHint(), KDialog::spacingHint(), "CommandConfigWidgetLayout");
   layout->setAlignment (Qt::AlignTop); 

   d = new HadifaxConfPrivate();
   d->configWidget = new HadifaxConfigUI (this, "configWidget");

   QString file = locate("data", "LICENSES/LGPL_V2");
   i18n("This plug in is distributed under the terms of the LGPL v2.");
   
   connect(d->configWidget->voiceButton, SIGNAL(clicked()), this, SLOT(voiceButton_clicked()));
   connect(d->configWidget->testButton, SIGNAL(clicked()), this, SLOT(testButton_clicked()));
   connect(d->configWidget, SIGNAL(changed(bool)), this, SLOT(configChanged (bool)));
   d->initializeVoices();
   d->setDefaults();
   layout->addWidget (d->configWidget);
}

/** Desctructor */
HadifaxConf::~HadifaxConf(){
   delete d;
   kdDebug() << "Running: HadifaxConf::~HadifaxConf()" << endl;
}

void HadifaxConf::load(KConfig *config, const QString &langGroup) {
   kdDebug() << "Running: HadifaxConf::load()" << endl;
   d->load (config, langGroup);
}

void HadifaxConf::save(KConfig *config, const QString &langGroup) {
   kdDebug() << "Running: HadifaxConf::save()" << endl;
   d->save (config, langGroup);
}

void HadifaxConf::defaults() {
   kdDebug() << "Running: HadifaxConf::defaults()" << endl;
   d->setDefaults();
}

void HadifaxConf::voiceButton_clicked () {
   KDialogBase *dialog = new KDialogBase (this, 0, true,
                                      i18n("Voice file - Hadifax plug in"),
                                      KDialogBase::Ok|KDialogBase::Cancel,
                                      KDialogBase::Ok, true);
   VoiceFileWidget *widget = new VoiceFileWidget(dialog);
   dialog->setMainWidget (widget);

   widget->femaleOption->setChecked(!d->configWidget->isMaleVoice());
   widget->maleOption->setChecked(d->configWidget->isMaleVoice());
   widget->voiceFileURL->setURL(d->configWidget->getVoiceFilename());
   widget->mbrola = d->defaultMbrolaExec;

   if (dialog->exec() == QDialog::Accepted) {
      d->configWidget->setVoice (widget->voiceFileURL->url(),
                                 widget->maleOption->isChecked());
      emit changed(true);
   }
   
   delete dialog;
}

void HadifaxConf::testButton_clicked () {
   // Speak a German sentence as hadifax is a German tts
   HadifaxSpeech::speak ("KDE ist eine moderne grafische Arbeitsumgebung fr Unix-Computer.",
                         d->configWidget->hadifaxURL->url(),  d->configWidget->isMaleVoice(),
                         d->configWidget->mbrolaURL->url(),   d->configWidget->getVoiceFilename(),
                         d->configWidget->volumeBox->value(), d->configWidget->timeBox->value(),
                         d->configWidget->frequencyBox->value());
}
