#include <qstring.h>
#include <qstringlist.h>

#include <kdebug.h>
#include <kconfig.h>

#include "hadifaxspeech.h"

#include "hadifaxproc.h"
#include "hadifaxproc.moc"

class HadifaxProcPrivate {
   friend class HadifaxProc;
   private:
      HadifaxProcPrivate (KConfig *config, const QString &langGroup) {
         load (config, langGroup);
      };

      ~HadifaxProcPrivate() {
      };

      void load(KConfig *config, const QString &langGroup) {
         config->setGroup(QString("Lang_")+langGroup);
         hadifax  = config->readEntry ("hadifaxExec",   QString::null);
         mbrola   = config->readEntry ("mbrolaExec",    QString::null);
         voice    = config->readEntry ("voice",         QString::null);
         gender   = config->readBoolEntry("gender",     false);
         volume   = config->readNumEntry ("volume",     100);
         time     = config->readNumEntry ("time",       100);
         pitch    = config->readNumEntry ("pitch",      100);
      };

      QString hadifax;
      QString mbrola;
      QString voice;
      bool gender;
      int volume;
      int time;
      int pitch;

      QWaitCondition runningCondition;
      QMutex runningMutex;
};

/** Constructor */
HadifaxProc::HadifaxProc( QObject* parent, const char* name, const QStringList &) : 
   PlugInProc( parent, name ){
   kdDebug() << "Running: HadifaxProc::HadifaxProc( QObject* parent, const char* name, const QStringList &args)" << endl;
   d = 0;
}

/** Desctructor */
HadifaxProc::~HadifaxProc(){
   kdDebug() << "Running: HadifaxProc::~HadifaxProc()" << endl;

   if (d != 0) {
      delete d;
      d = 0;
   }
}

/** Initializate the speech */
bool HadifaxProc::init(const QString &lang, KConfig *config){
   kdDebug() << "Running: HadifaxProc::init(const QString &lang, KConfig *config)" << endl;
   kdDebug() << "Initializing plug in: Hadifax" << endl;

   if (d == 0)
      d = new HadifaxProcPrivate(config, lang);
   else
      d->load(config, lang);
   return true;
}

/** Say a text
    text: The text to be speech
*/
void HadifaxProc::sayText(const QString &text) {
   kdDebug() << "Running: HadifaxProc::sayText(const QString &text)" << endl;
   kdDebug() << "Saying text: '" << text << "' using Hadifax plug in" << endl;
   if (d == 0)
      return;
   if (d->hadifax.isNull() || d->hadifax.isEmpty())
      return;
   if (d->mbrola.isNull() || d->mbrola.isEmpty())
      return;
   if (d->voice.isNull() || d->voice.isEmpty())
      return;
   
   d->runningMutex.lock();
   if (HadifaxSpeech::speak (text, d->hadifax, d->gender, d->mbrola, d->voice,
                   d->volume, d->time, d->pitch, this, SLOT(speakFinished())))
   {
      d->runningCondition.wait (&d->runningMutex);
   }
   d->runningMutex.unlock();
}

void HadifaxProc::speakFinished() {
   d->runningMutex.lock();
   d->runningCondition.wakeAll();
   d->runningMutex.unlock();
}

/**
 * Stop text
 */
void HadifaxProc::stopText(){
   kdDebug() << "Running: HadifaxProc::stopText()" << endl;
   // Bogus implementation by now
}
