/***************************************************************************
                          speech.cpp  -  description
                             -------------------
    begin                : Mon Okt 14 2002
    copyright            : (C) 2002 by Gunnar Schmi Dt
    email                : gunnar@schmi-dt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qfile.h>
#include <qstring.h>
#include <qregexp.h>

#include <kdebug.h>
#include <ktempfile.h>
#include <kstandarddirs.h>

#include <arts/kartsserver.h>
#include <arts/kartsdispatcher.h>
#include <arts/kplayobject.h>
#include <arts/kplayobjectfactory.h>

#include "hadifaxspeech.h"

HadifaxSpeech::HadifaxSpeech() {
   waveFile = QString::null;
}

HadifaxSpeech::~HadifaxSpeech() {
}

bool HadifaxSpeech::speak(QString text,
                          QString hadifax, bool isMale,
                          QString mbrola,  QString voice,
                          int volume, int time, int pitch,
                          QObject *receiver, const char *slot)
{
   if (   text.isNull()  || hadifax.isNull()  || mbrola.isNull()
       || text.isEmpty() || hadifax.isEmpty() || mbrola.isEmpty())
   {
      return false;
   }

   // create a temp file name for the wave file
   KTempFile tempFile (locateLocal("tmp", "hadifaxplugin-"), ".wav");
   QString waveFile = tempFile.file()->name();
   tempFile.close();

   QString hadifaxCommand = KShellProcess::quote(hadifax);
   if (isMale)
      hadifaxCommand += " -m";
   else
      hadifaxCommand += " -f";

   QString mbrolaCommand = KShellProcess::quote(mbrola);
   mbrolaCommand += " -e"; //Ignore fatal errors on unkown diphone
   mbrolaCommand += QString(" -v %1").arg(volume/100.0); // volume ratio
   mbrolaCommand += QString(" -f %1").arg(pitch/100.0);  // freqency ratio
   mbrolaCommand += QString(" -t %1").arg(time/100.0);   // time ratio
   mbrolaCommand += " " + KShellProcess::quote(voice);
   mbrolaCommand += " - " + KShellProcess::quote(waveFile);

   kdDebug() << "Hadifax plug in: " << hadifaxCommand << endl;
   kdDebug() << "Hadifax plug in: " << mbrolaCommand << endl;

   QString command = hadifaxCommand + "|" + mbrolaCommand;

   // create a new process
   HadifaxSpeech *speech = new HadifaxSpeech();
   speech->waveFile = waveFile;
   speech->process << command;
   connect(&speech->process, SIGNAL(processExited(KProcess *)), speech, SLOT(processExited(KProcess *)));
   connect(&speech->process, SIGNAL(wroteStdin(KProcess *)), speech, SLOT(wroteStdin(KProcess *)));
   if (receiver != 0)
      connect(speech, SIGNAL(speakFinished()), receiver, slot);
   speech->process.start(KProcess::NotifyOnExit, KProcess::Stdin);
   speech->process.writeStdin(text.latin1(), text.length());

   return true;
}

void HadifaxSpeech::wroteStdin(KProcess *) {
   process.closeStdin();
}

void HadifaxSpeech::processExited(KProcess *) {
   kdDebug() << "Hadifax plug in: using arts to play the sound" << endl;
   if (!waveFile.isNull() && !waveFile.isEmpty()) {
      // play the wave file
      KArtsDispatcher dispatcher;
      KArtsServer server;
      KDE::PlayObjectFactory factory (server.server());
      KDE::PlayObject* playobj = factory.createPlayObject (waveFile, true);
      playobj->play();
      while (playobj->state() == Arts::posPlaying)
         qApp->processEvents(500);
      playobj->halt();
      delete playobj;

      // clean up
      QFile::remove (waveFile);
   }

   emit speakFinished();
   delete this;
}

/***************************************************************************/

HadifaxSpeech::VoiceGender HadifaxSpeech::determineGender(QString mbrola, QString voice, QString *output) {
   QString command = mbrola + " -i " + voice + " - -";

   // create a new process
   HadifaxSpeech speech;
   speech.process << command;
   connect(&speech.process, SIGNAL(receivedStdout(KProcess *, char *, int)), &speech, SLOT(receivedStdout(KProcess *, char *, int)));
   connect(&speech.process, SIGNAL(receivedStderr(KProcess *, char *, int)), &speech, SLOT(receivedStderr(KProcess *, char *, int)));

   speech.stdOut = QString::null;
   speech.stdErr = QString::null;
   speech.process.start (KProcess::Block, KProcess::AllOutput);

   VoiceGender result;
   if (!speech.stdErr.isNull() && !speech.stdErr.isEmpty()) {
      if (output != 0)
         *output = speech.stdErr;
      result = NoVoice;
   }
   else {
      if (output != 0)
         *output = speech.stdOut;
      if (speech.stdOut.contains("female", false))
         result = FemaleGender;
      else if (speech.stdOut.contains("male", false))
         result = MaleGender;
      else
         result = NoGender;
   }
   
   return result;
}

void HadifaxSpeech::receivedStdout (KProcess *, char *buffer, int buflen) {
   stdOut += QString::fromLatin1(buffer, buflen);
}

void HadifaxSpeech::receivedStderr (KProcess *, char *buffer, int buflen) {
   stdErr += QString::fromLatin1(buffer, buflen);
}

#include "hadifaxspeech.moc"
