/***************************************************************************
                          hadifaxspeech.h  -  description
                             -------------------
    begin                : Mon Okt 14 2002
    copyright            : (C) 2002 by Gunnar Schmi Dt
    email                : gunnar@schmi-dt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HADIFAXSPEECH_H
#define HADIFAXSPEECH_H

#include <qobject.h>
#include <qstring.h>
#include <kprocess.h>
#include <qmutex.h>
#include <qwaitcondition.h>

/**
 * This class is used internally by HadifaxPlugin in order to do the actual
 * speaking.
 *@author Gunnar Schmi Dt
 */

class HadifaxSpeech : public QObject {
   Q_OBJECT
public:

   enum VoiceGender {
	MaleGender	=  2,
	FemaleGender	=  1,
	NoGender	=  0,
	NoVoice		= -1
    };

   /**
    * Speaks the given text.
    * @param text The text that shall be spoken
    * @param hadifax the hadifax executable
    * @param hadifaxParams parameters for hadifax
    * @param mbrola the mbrola executable
    * @param mbrolaParams parameters for mbrola
    * @return true if the text actually gets spoken
    */
   static bool speak (QString text,
                      QString hadifax, bool isMale,
                      QString mbrola,  QString voice,
                      int volume, int time, int pitch,
                      QObject *receiver=0, const char *slot=0);

   /**
    * Determines whether the voice file is male or female.
    * @param mbrola the mbrola executable
    * @param voice the voice file
    * @param output the output of mbrola will be written into this QString*
    * @return HadifaxSpeech::MaleGender if the voice is male,
    *         HadifaxSpeech::FemaleGender if the voice is female,
    *         HadifaxSpeech::NoGender if the gender cannot be determined,
    *         HadifaxSpeech::NoVoice if there is an error in the voice file
    */
   static VoiceGender determineGender(QString mbrola, QString voice, QString *output = 0);

private:
   /** Constructor */
   HadifaxSpeech();
public:
   /** Destructor */
   ~HadifaxSpeech();

signals:
   void speakFinished();

private slots:
   void wroteStdin (KProcess *p);
   void processExited (KProcess *p);
   void receivedStdout (KProcess *proc, char *buffer, int buflen);
   void receivedStderr (KProcess *proc, char *buffer, int buflen);

private:
   KShellProcess process;
   QString waveFile;
   QString stdOut;
   QString stdErr;
};

#endif
